package HCT::Shell;

our $VERSION = '0.0.1';

use strict;
use warnings;

use Text::ParseWords ();

use HCT::Constants;

our $STANDARD_PROMPT ||= '> ';
our $ENVIRONMENT = 'HCT::Shell::Env';
our $SUPPRESS_READLINE;
our $TERM;

our $PRINT_ORNAMENTING ||= FALSE;

sub init
{
	my $class = shift;

	require Term::ReadLine;

	if (!$TERM or $TERM->ReadLine eq 'Term::ReadLine::Stub') {
	   	$TERM = Term::ReadLine->new ('HCT');
	}

	if ($TERM->ReadLine eq 'Term::ReadLine::Gnu') {
    	my $attributes = $TERM->Attribs;

		# attributes
    	$attributes->{'attempted_completion_function'} = sub {	print 2;	};
		$attributes->ornaments ('md,me,,');
	}
}

sub run
{
	my $class = shift;

    $SUPPRESS_READLINE = ! -t STDIN unless defined $SUPPRESS_READLINE;

    my $commandline = "";
	my $prompt = $STANDARD_PROMPT;

	local ($^W) = TRUE;
	unless ($SUPPRESS_READLINE) {
		print "Welcome to " and HCT::HCT->print_version ();
		HCT::Shell->init ();

		my $h = select STDERR;
		$| = 1;
		select STDOUT;
		$| = 1;
		select $h;
    }

	my $continuation = "";

	SHELLCOMMAND: while () {
		if ($SUPPRESS_READLINE) {
			print $prompt;
			last SHELLCOMMAND unless defined ($_ = <>);
	  		chomp;
		} else {
			last SHELLCOMMAND unless
				defined ($_ = $TERM->readline($prompt, $commandline));
		}
		
		# continue?
		$_ = "${continuation}$_"
			if $continuation;

		s/^\s+//;
		next SHELLCOMMAND if /^$/;

		$_ = 'help' if (/^\s*\?/);

		if (s/\\$//s) {
			chomp;
			$continuation = $_;
			$prompt = " " x length ($STANDARD_PROMPT);
		}

		elsif (/./) {
			my @line;
			if ($] < 5.00322) {
				@line = split;
			} else {
				eval { @line = Text::ParseWords::shellwords ($_) };
				HCT::DEBUG ($@), next SHELLCOMMAND if $@;
				HCT::DEBUG ("Text::ParseWords could not parse the line [$_]"), next SHELLCOMMAND unless @line;
			}
			my $command = shift @line;
			eval { $ENVIRONMENT->$command (@line) };
			if (/^(?:quit|bye|exit)$/i) {
	    		last SHELLCOMMAND;
			}

			$continuation = "";
			$prompt = $STANDARD_PROMPT;
		}
	}

    exit;
}

sub print_ornamented {
    my ($class, $what, $ornament) = @_;
    my $longest = 0;

    return unless defined $what;

    if ($CPAN::Config->{term_is_latin}){
        $what =~ s{([\xC0-\xDF])([\x80-\xBF])}{chr(ord($1)<<6&0xC0|ord($2)&0x3F)}eg;
    }

	if ($PRINT_ORNAMENTING) {
		unless (defined &color) {
	    	if ($CPAN::META->has_inst ("Term::ANSIColor")) {
				import Term::ANSIColor "color";
		    } else {
				*color = sub { return "" };
			}
		}

		my $line;
		for $line (split /\n/, $what) {
	    	$longest = length($line) if length($line) > $longest;
		}

		my $sprintf = "%-" . $longest . "s";

		while ($what){
	    	$what =~ s/(.*\n?)//m;
	    	my $line = $1;
	    	last unless $line;
	    	my($nl) = chomp $line ? "\n" : "";
	    	print color($ornament), sprintf($sprintf,$line), color("reset"), $nl;
		}

    } else {
        # chomp $what;
        # $what .= "\n"; # newlines unless $PRINT_ORNAMENTING
		print $what;
    }
}

sub warning
{
	my ($class, $what) = @_;
    $class->print_ornamented ($what, 'bold red on_yellow');
}

package HCT::Shell::Env;

our $AUTOLOAD;

sub AUTOLOAD
{
 	my $autoload = $AUTOLOAD;
	my $class = shift;
	$autoload =~ s/.*:://;
	HCT::Shell->warning ("Unknown command '$autoload'. Type ? for help.\n");
}

sub exit {}
sub bye  {}
sub quit {}

sub new
{
	my $class = shift;
	my ($what, @details) = (shift, @_);
	
	if ($what eq 'project') {
		if (scalar (@details) == 0) {
			HCT::Shell->warning ("To create new project need more details.\n");
			return;
		}
		@ARGV = @details;
		&HCT::HCT::do_main_loop;
	}
}

1;
