# Copyright 2008, 2009
# Slade Maurer
#
# This file is part of HCT, the Hardware Complexity Tool
#
# HCT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# HCT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with HCT.  If not, see <http://www.gnu.org/licenses/>.

package HCT::Std::IO;

our $VERSION = '0.0.1';

use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT = qw(is_interactive);

use Scalar::Util qw(openhandle);

use HCT::Constants;

sub is_interactive
{
    my ($output_handle) = (@_, select); # Default to default output handle

    # Not interactive if output is not to terminal...
    return FALSE if not -t $output_handle;

    # If *ARGV is opened, we're interactive if...
    if (openhandle *ARGV) {
        # ...it's currently opened to the magic '-' file
        return -t *STDIN if defined $ARGV && $ARGV eq '-';

        # ...it's at end-of-file and the next file is the magic '-' file
        return @ARGV > 0 && $ARGV[0] eq '-' && -t *STDIN if eof *ARGV;

        # ...it's directly attached to the terminal 
        return -t *ARGV;
    }

    # If *ARGV isn't opened, it will be interactive if *STDIN is attached to a terminal.
    else {
        return -t *STDIN;
    }
}

#___________________________________________________________________________

package HCT::Std::IO::Handle;

$VERSION = '0.0.1';

use base 'IO::Handle';

1;
