# Bar.pm
#
# Copyright 2008, 2009
# Slade Maurer
#
# This file is part of HCT, the Hardware Complexity Tool
#
# HCT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# HCT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with HCT.  If not, see <http://www.gnu.org/licenses/>.

package Term::Bar;

use strict;

# True if we're on Win32.
use constant ON_WINDOWS => (($^O =~ /^MSWin/) or ($^O eq "Windows_NT")) ? 1 :
    ($ENV{'SHELL'}) ? 0  # make Cygwin look like Unix                      
    : 0;

use constant TRUE   => 1;
use constant FALSE  => 0;

sub new
{
    my $inv = shift;
    my $class = ref($inv) || $inv;
    my $self = {
		wrapper => [],
		separator => "",
		filler => " ",
		colorize => FALSE, 
		@_,
		items => [],
    };
    bless $self, $class;
    return $self;
}

sub item
{
    my $self = shift;
    my $item = HCT::Term::Bar::Item->new(
		colorize => $self->{'colorize'},
		filler => $self->{'filler'},
		@_
	);
    push @{ $self->{'items'} }, $item;
    $item;
}

sub represent
{
    my $self = shift;
    print $self->get;
}

sub get
{
    my $self = shift;
    my @items = ();
    my $bar = "";

    foreach my $item (@{ $self->{'items'} }) {
		push @items, $item->get;
    }

    for (my $i=0; $i<scalar(@items); $i++) {
		$bar .= $items[$i];
		if ( $i != $#items ) {
			$bar .= $self->{'separator'};
		}
    }

    return $bar;
}

package HCT::Term::Bar::Item;

use constant LEFT  => 0;
use constant RIGHT => 1;

sub new
{
    my $inv = shift;
    my $class = ref($inv) || $inv;
    my $self = {
	length => 0,
		text => '',
		align => 'right',
		wrapper => [],
		filler_wrapper => [],
		@_
    };

    bless $self, $class;
    $self->validate;
    $self;
}

sub set_wrapper { $_[0]->{'wrapper'} = $_[1] }

sub validate
{
    my $self = shift;
    if (length($self->{'text'}) > $self->{'length'}) {
	
    }
    
    if (length($self->{'filler'}) < 1) {
	
    }
}

sub _wrap
{
    my $self = shift;
    my ($str, $wrapper) = (shift, shift);
    my $numof_elem = scalar(@$wrapper);
    my $step = length($str) / ($numof_elem-1);
    my ($pos,$i) = (0,0);
    while ($i < $numof_elem) {
		my $txt = $wrapper->[$i];
		if (defined $txt && length($txt) > 0) {
	    	substr($str, $pos, 0, $txt);
	    	$pos += length($txt);
		}
		$pos += $step;
		$i++;
    }
    return $str;
}

sub _get_filler
{
    my $self = shift;
    my $filler = ['',''];

    return $filler if int($self->{'length'} - length($self->{'text'})) == 0;
    my $content = $self->{'filler'} x ($self->{'length'} - length($self->{'text'}));

    if (defined $self->{'filler_wrapper'} && scalar(@{$self->{'filler_wrapper'}}>1)) {
        $content = $self->_wrap($content, $self->{'filler_wrapper'});
    }

    if    ($self->{'align'} eq 'right') { $filler->[RIGHT] = $content }
    elsif ($self->{'align'} eq 'left' ) { $filler->[LEFT ] = $content }
    elsif ($self->{'align'} eq 'center') {
	#...
    }

    return $filler;
}

sub get
{
    my $self = shift;
    my $content = '';

    my $filler = $self->_get_filler;
    $content .= join("" => @$filler) . $self->{'text'} if $self->{'align'} eq 'right';
    # ...

    if (defined $self->{'wrapper'} && scalar(@{$self->{'wrapper'}}>1)) {
        $content = $self->_wrap($content, $self->{'wrapper'});
    }

    return $content;
}

1;

